/*
	DataButtons.cc

   Modified by Gary White (gary@apptools.com)
   October 16, 1998
	---------------------------------------------------------------------
   November 3, 1998, added the InitRowset method to the rowState object
   based upon code by Ken Mayer to allow changing the form's rowset.

   				Thanks Ken!
	---------------------------------------------------------------------
	This is a modified version of the DataButtons.cc that shipped with
   VdB7.  The original version offered a RowState object which displayed
   the current state of the form's rowset.  Virtually all the functions
   in this file called a RefreshRowstate method which traversed all the
   controls on the form to see if any of them were Rowstate objects and,
   if so, to update the text displayed.  On a large form, this could be
   time consuming.  The other drawback was that you had to remember to 
   call this function from your menu procedures if you wanted the 
   rowstate display to update.

   This version uses a calculated field, datalinked to an entryfield to
   display rowset.state information.  Updating is all automatic and thus
   dispenses with the need to call the RefreshRowstate function.  This
   should offer much better performance.  The RefreshRowstate function
   was left in the file, but all code was removed from it.  This was done
   so that any existing code using the original version that may have 
   called the RefreshRowstate function would still operate, but would 
   still get the performance improvements.

   Portions of this file are Copyright (c) 1997-1998, 
   	Borland International, Inc.

*/
//------------------------------------------------------------------------
//
//  DataButtons.CC  
//
//  Custom control library for data navigation and editing.
//
//  Control         Function 
//  ----------      -----------------------------------------------
//  BarDataVCR      Container consisting of BitmapFirst,
//                  BitmapPrevious, BitmapNext and BitmapLast.
//
//  BarDataEdit     Container consisting of BitmapAppend,
//                  BitmapDelete, BitmapSave, BitmapAbandon, 
//                  BitmapFilter and BitmapLocate.
//
//  BitmapAppend    Add a new row.
//               
//  BitmapDelete    Remove current row without confirmation.
//
//  BitmapSave      Save changes to current row.
//
//  BitmapAbandon   Abandon changes to current row.
//
//  BitmapFilter    Switch to or run filter by form. 
//
//  BitmapLocate    Switch to or run query by form.
//
//  BitmapEdit      Switch to edit state from browse state.
//                  This has no effect when rowset.autoEdit 
//                  is true.
//
//  BitmapFirst     Move to first row.
//
//  BitmapPrevious  Move to previous row.
//
//  BitmapNext      Move to next row.
//
//  BitmapLast      Move to last row.
//
//  RowState        Displays current rowset state as: Closed,
//                  Read Only, Update, Append, Filter or Locate.
//
//  All controls operate on the current form rowset. 
//
//  All DataButton controls except the current control are hidden 
//  when switching to a filter or locate state. All DataButtons are 
//  shown when switching out of a filter or locate state.
//
//  Button Control    Same Function As
//  --------------    ----------------
//  ButtonAppend      BitmapAppend
//  ButtonDelete      BitmapDelete
//  ButtonSave        BitmapSave
//  ButtonAbandon     BitmapAbandon
//  ButtonFilter      BitmapFilter
//  ButtonLocate      BitmapLocate
//  ButtonEdit        BitmapEdit
//  ButtonFirst       BitmapFirst
//  ButtonPrevious    BitmapPrevious
//  ButtonNext        BitmapNext
//  ButtonLast        BitmapLast
//
//  Visual dBASE Samples Group
//  $Revision:   1.7  $
//
//  Copyright (c) 1997-1998, Borland International, Inc. 
//  All rights reserved.
//
//------------------------------------------------------------------------

#define DATACONTROLS1 "BITMAPAPPEND,BITMAPDELETE,BITMAPSAVE,BITMAPABANDON,BITMAPLOCATE,BITMAPFILTER,BITMAPEDIT"
#define DATACONTROLS2 ",BUTTONAPPEND,BUTTONDELETE,BUTTONSAVE,BUTTONABANDON,BUTTONLOCATE,BUTTONFILTER,BUTTONEDIT"
#define DATACONTROLS3 ",BITMAPFIRST,BITMAPPREVIOUS,BITMAPNEXT,BITMAPLAST"
#define DATACONTROLS4 ",BUTTONFIRST,BUTTONPREVIOUS,BUTTONNEXT,BUTTONLAST"
#define DATACONTROLS5 ",BARDATAEDIT,BARDATAVCR"

// Bitmap Buttons

class BitmapData(ParentObj) of PushButton(ParentObj) custom 
   SET TALK OFF
   with ( this )
      onOpen   := class::dataButton_onOpen
      speedBar := true
      text     := null
      width    := 32
      height   := 32
      metric   := 6 // pixels
      fontSize := 8
   endwith

   function dataButton_onOpen
      private rType
      rType = this.form.rowset
      this.visible = ( TYPE( "rType" ) <> "U" )
   return ( this.visible )

   function toggleSearch( current )  
      local bSearch, sControlList
      local vProp

      bSearch = ( this.form.rowset.state == 4 ) OR ;
                ( this.form.rowset.state == 5 )
      sControlList = UPPER( DATACONTROLS1 + DATACONTROLS2 + ;
                            DATACONTROLS3 + DATACONTROLS4 )
      sBarList     = UPPER( DATACONTROLS5 )
                    
      for i = 1 to this.form.elements.size

          if ( UPPER( this.form.elements[i].className ) $ sControlList )
             if ( this.form.elements[i] <> current ) 
                this.form.elements[i].enabled = ( NOT bSearch ) 
             endif      
          endif

          if ( UPPER( this.form.elements[i].className ) $ sBarList )
             this.form.elements[i].barToggle( bSearch, current )
          endif
      next

   return ( bSearch )

   function refreshRowState      
	// Left in to keep from breaking old code
   return
endclass


class BitmapAppend( ParentObj ) of BitmapData( ParentObj ) custom 
   with ( this )
      upBitmap := "RESOURCE:2 PS_APPEND"
      speedTip := "Add Row"
      onClick  := { ; this.form.rowset.beginAppend() }
   endwith
endclass

class BitmapDelete( ParentObj ) of BitmapData( ParentObj ) custom 
   with ( this )
      upBitmap := "RESOURCE:2 PS_DELETE"
      speedTip := "Delete Row"
      onClick  := class::BitmapDelete_onClick
   endwith

   function BitmapDelete_onClick
      local bDelete
      bDelete = false

      if ( not this.form.rowset.endOfSet )

         if ( MSGBOX("You are about to delete the current row." ;
              + CHR(13) ;
              + "Click Yes to delete the current row.", ;
              "Alert", ;
              4 ) == 6 )

            bDelete := this.form.rowset.delete()

            if ( this.form.rowset.endOfSet )
              this.form.rowset.last()
            endif

         endif

      endif

   return ( bDelete )

endclass

class BitmapSave( ParentObj ) of BitmapData( ParentObj ) custom 
   with ( this )
      upBitmap := "RESOURCE:2 PS_SAVE"
      speedTip := "Save Row"
      onClick  := { ; this.form.rowset.save() }
   endwith
endclass
                               
class BitmapAbandon( ParentObj ) of BitmapData( ParentObj ) custom 
   with ( this )
      upBitmap := "RESOURCE:2 PS_ABANDON"
      speedTip := "Abandon"
      onClick  := { ; this.form.rowset.abandon() }
   endwith
endclass

class BitmapLocate(ParentObj) of BitmapData( ParentObj ) custom
   with ( this )
      upBitmap := "RESOURCE:2 PS_LOCATE"
      speedTip := "Find Row"
      onClick  := class::bitmapLocate_onClick
   endwith

   function bitmapLocate_onClick
      // Change image and toggle other controls
      // when entering and exiting search mode.

      if ( this.form.rowset.state == 5 ) 
         this.upBitmap = "RESOURCE:2 PS_LOCATE"
         this.speedTip = "Find Row"
         this.form.rowset.applyLocate()
      else
         this.upBitmap = "RESOURCE:2 PS_RUN"
         this.speedTip = "Start Search" 
         this.form.rowset.beginLocate()
      endif
      super::toggleSearch(this)
   return ( this.form.rowset.state )
endclass

class BitmapFilter( ParentObj ) of BitmapData( ParentObj ) custom 
   with ( this )
      upBitmap := "RESOURCE:2 PS_FILTER"
      speedTip := "Filter Rowset"
      onClick  := class::bFilter_onClick
   endwith

   function bFilter_onClick
      // Change image and toggle other controls
      // when entering and exiting search mode.

      if ( this.form.rowset.state == 4 ) 
         this.upBitmap = "RESOURCE:2 PS_FILTER"
         this.speedTip = "Filter Rowset"
         this.form.rowset.applyFilter()     
      else
         this.upBitmap = "RESOURCE:2 PS_RUN"
         this.speedTip = "Apply Filter"
         this.form.rowset.beginFilter()
      endif
      super::toggleSearch(this)
   return ( this.form.rowset.state )
endclass

class BitmapEdit(ParentObj) of BitmapData( ParentObj ) custom 
   with ( this )
      upBitmap := "RESOURCE:2 PS_EDIT"
      speedTip := "Edit Row"
      onClick  := { ; this.form.rowset.beginEdit() }
   endwith
endclass


class BitmapFirst( ParentObj ) of BitmapData( ParentObj ) custom 
   with ( this )
      upBitmap := "RESOURCE:2 PS_FIRST"
      speedTip := "First Row"
      onClick  := { ; this.form.rowset.first() }
   endwith
endclass

class BitmapPrevious( ParentObj ) of BitmapData( ParentObj ) custom 
   with ( this )
      upBitmap := "RESOURCE:2 PS_PREV"
      speedTip := "Previous Row"
      onClick  := { ; if ( NOT this.form.rowset.next(-1)) ;
                    ;    this.form.rowset.next() ; 
                    ; endif }
   endwith
endclass

class BitmapNext( ParentObj ) of BitmapData( ParentObj ) custom 
   with ( this )
      upBitmap := "RESOURCE:2 PS_NEXT"
      speedTip := "Next Row"
      onClick  := { ; if ( NOT this.form.rowset.next() ) ;
                    ;    this.form.rowset.next(-1) ;
                    ; endif }
   endwith
endclass

class BitmapLast(ParentObj) of BitmapData( ParentObj ) custom 
   with ( this )
      upBitmap := "RESOURCE:2 PS_LAST"
      speedTip := "Last Row"
      onClick  := { ; this.form.rowset.last() }
   endwith
endclass

class BarDataEdit( ParentObj, Name ) of Container( ParentObj, Name ) custom
   SET TALK OFF
   with ( this )
      width  := 192
      height := 32
      metric := 6 // pixels
      borderStyle := 3 // none
   endwith

   this.bitmapAppend = new BitmapAppend( this )
                            
   this.bitmapDelete = new BitmapDelete( this )
   with ( this.bitmapDelete )
      left     := 32
      width    := 32
      height   := 32
      fontSize := 8
      metric   := 6 // pixels
   endwith

   this.bitmapSave = new BitmapSave( this )
   with ( this.bitmapSave )
      left     := 64
      width    := 32
      height   := 32
      fontSize := 8
      metric   := 6 // pixels
   endwith
        
   this.bitmapAbandon = new BitmapAbandon( this )
   with ( this.bitmapAbandon )
      left     := 96
      width    := 32
      height   := 32
      fontSize := 8
      metric   := 6 // pixels
   endwith

   this.bitmapFilter = new BitmapFilter( this )
   with ( this.bitmapFilter )
      left     := 128
      width    := 32
      height   := 32
      fontSize := 8
      metric   := 6 // pixels
   endwith
        
   this.bitmapLocate = new BitmapLocate( this )
   with ( this.bitmapLocate )
      left     := 160
      width    := 32
      height   := 32
      fontSize := 8
      metric   := 6 // pixels
   endwith

   function barToggle( bSearch, current )
      with ( this )
         bitmapAppend.enabled  := ( not bSearch )
         bitmapDelete.enabled  := ( not bSearch )
         bitmapSave.enabled    := ( not bSearch )
         bitmapAbandon.enabled := ( not bSearch )
      endwith
      if ( current <> this.bitmapFilter )
         this.BitmapFilter.enabled := ( not bSearch )
      endif
      if ( current <> this.bitmapLocate )
         this.BitmapLocate.enabled := ( not bSearch )
      endif
   return ( bSearch )

endclass


class BarDataVCR( ParentObj, Name ) of Container( ParentObj, Name ) custom
   SET TALK OFF
   with ( this )
      width  := 128
      height := 32
      metric := 6 // pixels
      borderStyle := 3 // none
   endwith

   this.bitmapFirst = new BitmapFirst( this )

   this.bitmapPrevious = new BitmapPrevious( this )
   with ( this.bitmapPrevious )
      left     := 32
      width    := 32
      height   := 32
      fontSize := 8
      metric   := 6 // pixels
   endwith

   this.bitmapNext = new BitmapNext( this )
   with ( this.bitmapNext )
      left     := 64
      width    := 32
      height   := 32
      fontSize := 8
      metric   := 6 // pixels
   endwith
        
   this.bitmapLast = new BitmapLast( this )
   with ( this.bitmapLast )
      left     := 96
      width    := 32
      height   := 32
      fontSize := 8
      metric   := 6 // pixels
   endwith

   function barToggle( bSearch, current )
      with ( this )
         bitmapFirst.enabled    := ( not bSearch )
         bitmapPrevious.enabled := ( not bSearch )
         bitmapNext.enabled     := ( not bSearch )
         bitmapLast.enabled     := ( not bSearch )
      endwith
   return ( bSearch )

endclass




class RowState(parentObj, name) of ENTRYFIELD(parentObj, name) custom
/*
	This RowState object displays the current state of a rowset.  Unlike the
   original one shipped with VdB, this one uses an entryfield, datalinked to
   a calculated field.  This eliminates the necessity of manually refreshing
   it.  This was a large task in the original, since it traversed all the 
   objects on the form to see if any of them were rowstate objects.  On a 
   large form, this could be rather slow.  With this one, the calc field 
   handles the update automatically.

	This should function just fine with any existing code using the original
   unless you manually manipulated the text property of the original version.
   You don't need to force a refresh in any of your functions.

   Thanks to Ken Mayer, who came up with the idea and code of the initRowset
   method which allows you to change rowsets for an existing rowstate ojbect.

   // Additional code by Ken Mayer
   initRowset method was added -- the reasoning is:
      in many of my forms, I do a parent/child relationship,
      and switch rows based on which table has focus, by
      setting the form's rowset property. It is absurd
      to have to do a bunch of extra work to get the rowState
      class here to keep up. This method will make sure that
      if the calculated field does not exit, it gets created
      (calling this object's onOpen event), and then it
      makes sure the dataLink is correct. It works in the 
      testing I've done, pretty well ...

*/

	// hide some properties that aren't appropriate and our methods
	protect datalink, function, maxLength, memoEditor, phoneticLink, picture,;
   	selectAll, validErrorMsg, validRequired, tabStop, key, onChange,;
      valid, enabled, helpFile, helpId, mousePointer, statusMessage, tabStop,;
      colorHighLight, onOpen, onGotFocus

   with (this)
      onOpen = class::rowState_onOpen
      when = {;return false}						// we don't want focus
      onGotFocus = {;this.keyboard("{Tab}")}	// just in case
      height = 0.8182
      left = 11.7143
      top = 0.6818
      width = 8
      metric = 0	// Chars
      colorNormal = "BtnText/BtnFace"
      fontSize = 9
      fontBold = true
      fontItalic = true
      value = "RowState"
      mousePointer = 1	// Arrow
      borderStyle = 1	// Raised
   endwith

   this.rowset = null

   // Added by Ken:
   function initRowset
      // The idea is if you are using a second (or more)
      // rowset on the form, but one set of buttons
      // to manipulate any/all rowsets, you need
      // to make sure that the "current" rowset
      // has the calc field necessary to update
      // this control:

      // check for calc field, if it already exists,
      // there's nothing to do:
      this.rowsetDisplayed = form.rowset
      if type('this.rowsetDisplayed.fields["gwRowstate"]') == "O"
         this.dataLink = this.rowsetDisplayed.fields["gwRowstate"]
      else
         class::rowState_onOpen()
      endif
      return

   function rowState_onOpen
      if type("form.rowset") == "O"	// if the form has a rowset

      	// allow the rowset to be something other than the form.rowset
         // if you want to display another rowset, just set
         // 		rowstate.rowset = rowsetToDisplay
         // in the form's onOpen event.
      	this.rowsetDisplayed = iif( empty(this.rowset), form.rowset, this.rowset )

         // if we haven't already defined the calc field
         if type('this.rowsetDisplayed.fields["gwRowstate"]') # "O"
         	// do it here
            local f
            f = new field()			// create it
            f.states = new array()	// build an array with the possible states
            f.states.add("Closed")
            f.states.add("Browse")
            f.states.add("Edit")
            f.states.add("Append")
            f.states.add("Filter")
            f.states.add("Locate")
            f.fieldName = "gwRowstate"
            f.beforegetvalue={;return this.states[ this.parent.parent.state + 1 ]}
            this.rowsetDisplayed.fields.add(f)
         endif
         this.datalink = this.rowsetDisplayed.fields["gwRowstate"]
      else
         this.visible = false		// if no rowset, hide the display
      endif
      return
endclass

// Push Buttons

class ButtonData( ParentObj ) of PushButton( ParentObj ) custom 
   SET TALK OFF
   with ( this )
      onOpen   := class::buttonData_onOpen
      fontSize := 8
      fontBold := false 
      width    := 10
      height   := 1.2
      metric   := 0
   endwith

   function buttonData_onOpen
      private rType
      rType = this.form.rowset
      this.visible = ( TYPE( "rType" ) <> "U" )
   return ( this.visible )

   function toggleSearch( current )  
      local bSearch, sControlList
      bSearch = ( this.form.rowset.state == 4 ) OR ;
                ( this.form.rowset.state == 5 )
      sControlList = DATACONTROLS1 + DATACONTROLS2 + ;
                     DATACONTROLS3 + DATACONTROLS4

      for i = 1 to this.parent.elements.size
          if ( this.parent.elements[i].className $ sControlList )
             if ( this.parent.elements[i] <> current ) 
                this.parent.elements[i].visible = ( not bSearch ) 
             endif
          endif
      next
                    
   return ( bSearch )

   function refreshRowState
   // this is left in to keep from breaking old code
   return

endclass

class ButtonAppend( ParentObj ) of ButtonData( ParentObj ) custom
   with ( this )
      text    := "Add"
      onClick := { ; this.form.rowset.beginAppend() }
   endwith
endclass

class ButtonDelete(ParentObj ) of ButtonData( ParentObj ) custom 
   with ( this )
      text    := "Delete"
      onClick := class::buttonDelete_onClick
   endwith

   function buttonDelete_onClick
      local bDelete
      bDelete = false

      if ( not this.form.rowset.endOfSet )

         if ( MSGBOX("You are about to delete the current row." ;
              + CHR(13) ;
              + "Click Yes to delete the current row.", ;
              "Alert", ;
              4 ) == 6 )

            bDelete := this.form.rowset.delete()

            if (this.form.rowset.endOfSet)
               this.form.rowset.last()
            endif

         endif

      endif

   return ( bDelete )

endclass

class ButtonSave( ParentObj ) of ButtonData( ParentObj ) custom
   with ( this )
       text    := "Save"
       onClick := { ; this.form.rowset.save() }
   endwith
endclass

class ButtonAbandon( ParentObj ) of ButtonData( ParentObj ) custom
   with ( this )
      text    := "Abandon"
      onClick := { ; this.form.rowset.abandon() }
   endwith
endclass

class ButtonLocate( ParentObj ) of ButtonData( ParentObj ) custom
   with ( this )
      text    := "Locate"
      onClick := class::buttonLocate_onClick
   endwith
 
   function buttonLocate_onClick
      // Change text and toggle other controls
      // when entering and exiting search mode.

      if ( this.form.rowset.state == 5 ) 
         this.text = "Locate"
         this.form.rowset.applyLocate()
      else 
         this.text = "Start Search"
         this.form.rowset.beginLocate()
      endif
      super::toggleSearch(this)
   return ( this.form.rowset.state )
endclass

class ButtonFilter( ParentObj ) of ButtonData( ParentObj ) custom
   with ( this )
      text    := "Filter"
      onClick := class::buttonFilter_onClick
   endwith

   function buttonFilter_onClick 
      // Change text and toggle other controls
      // when entering and exiting search mode.

      if ( this.form.rowset.state == 4 ) 
         this.text = "Filter"
         this.form.rowset.applyFilter()      
      else 
         this.text = "Start Search"
         this.form.rowset.beginFilter()
      endif
      super::toggleSearch(this)
   return ( this.form.rowset.state )
endclass


class ButtonEdit( ParentObj ) of ButtonData( ParentObj ) custom
   with ( this )
      text    := "Edit"
      onClick := { ; this.form.rowset.beginEdit() }
   endwith
endclass

class ButtonFirst( ParentObj ) of ButtonData( ParentObj ) custom
   with ( this )
      text    := "First"
      onClick := { ; this.form.rowset.first() }
   endwith
endclass

class ButtonPrevious( ParentObj ) of ButtonData( ParentObj ) custom
   with ( this )
      text    := "Previous"
      onClick := { ; if ( NOT this.form.rowset.next(-1) ) ;
                   ;    this.form.rowset.next() ; 
                   ; endif }
   endwith
endclass

class ButtonNext( ParentObj ) of ButtonData( ParentObj ) custom
   with ( this )
      text    := "Next"
      onClick := { ; if ( NOT this.form.rowset.next() ) ;
                   ;    this.form.rowset.next(-1) ; 
                   ; endif }
   endwith
endclass

class ButtonLast( ParentObj ) of ButtonData( ParentObj ) custom
   with ( this )
      text    := "Last"
      onClick := { ; this.form.rowset.last() }
   endwith
endclass


